# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mindinsight_lineage.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='mindinsight_lineage.proto',
  package='mindinsight',
  syntax='proto2',
  serialized_pb=_b('\n\x19mindinsight_lineage.proto\x12\x0bmindinsight\"\xab\x02\n\x0cLineageEvent\x12\x11\n\twall_time\x18\x01 \x02(\x01\x12\x0c\n\x04step\x18\x02 \x01(\x03\x12\x11\n\x07version\x18\x03 \x01(\tH\x00\x12\x32\n\rtrain_lineage\x18\x06 \x01(\x0b\x32\x19.mindinsight.TrainLineageH\x00\x12<\n\x12\x65valuation_lineage\x18\x07 \x01(\x0b\x32\x1e.mindinsight.EvaluationLineageH\x00\x12\x32\n\rdataset_graph\x18\t \x01(\x0b\x32\x19.mindinsight.DatasetGraphH\x00\x12\x39\n\x11user_defined_info\x18\n \x01(\x0b\x32\x1c.mindinsight.UserDefinedInfoH\x00\x42\x06\n\x04what\"\x9a\x04\n\x0fUserDefinedInfo\x12/\n\tuser_info\x18\x01 \x03(\x0b\x32\x1c.mindinsight.UserDefinedInfo\x12;\n\x08map_dict\x18\x02 \x03(\x0b\x32).mindinsight.UserDefinedInfo.MapDictEntry\x12=\n\tmap_int32\x18\x03 \x03(\x0b\x32*.mindinsight.UserDefinedInfo.MapInt32Entry\x12\x39\n\x07map_str\x18\x04 \x03(\x0b\x32(.mindinsight.UserDefinedInfo.MapStrEntry\x12?\n\nmap_double\x18\x05 \x03(\x0b\x32+.mindinsight.UserDefinedInfo.MapDoubleEntry\x1aL\n\x0cMapDictEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12+\n\x05value\x18\x02 \x01(\x0b\x32\x1c.mindinsight.UserDefinedInfo:\x02\x38\x01\x1a/\n\rMapInt32Entry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a-\n\x0bMapStrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x30\n\x0eMapDoubleEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\"\xb6\x04\n\x0cTrainLineage\x12\x43\n\x10hyper_parameters\x18\x01 \x01(\x0b\x32).mindinsight.TrainLineage.HyperParameters\x12=\n\rtrain_dataset\x18\x02 \x01(\x0b\x32&.mindinsight.TrainLineage.TrainDataset\x12\x36\n\talgorithm\x18\x03 \x01(\x0b\x32#.mindinsight.TrainLineage.Algorithm\x12.\n\x05model\x18\x04 \x01(\x0b\x32\x1f.mindinsight.TrainLineage.Model\x1a\xa0\x01\n\x0fHyperParameters\x12\x11\n\toptimizer\x18\x01 \x01(\t\x12\x15\n\rlearning_rate\x18\x02 \x01(\x02\x12\x15\n\rloss_function\x18\x03 \x01(\t\x12\r\n\x05\x65poch\x18\x04 \x01(\x05\x12\x15\n\rparallel_mode\x18\x05 \x01(\t\x12\x12\n\ndevice_num\x18\x06 \x01(\x05\x12\x12\n\nbatch_size\x18\x08 \x01(\x05\x1a\x46\n\x0cTrainDataset\x12\x1a\n\x12train_dataset_path\x18\x01 \x01(\t\x12\x1a\n\x12train_dataset_size\x18\x02 \x01(\x05\x1a*\n\tAlgorithm\x12\x0f\n\x07network\x18\x01 \x01(\t\x12\x0c\n\x04loss\x18\x02 \x01(\x02\x1a#\n\x05Model\x12\x0c\n\x04path\x18\x03 \x01(\t\x12\x0c\n\x04size\x18\x04 \x01(\x03\"\xaf\x01\n\x11\x45valuationLineage\x12\x0e\n\x06metric\x18\x02 \x01(\t\x12\x42\n\rvalid_dataset\x18\x03 \x01(\x0b\x32+.mindinsight.EvaluationLineage.ValidDataset\x1a\x46\n\x0cValidDataset\x12\x1a\n\x12valid_dataset_path\x18\x01 \x01(\t\x12\x1a\n\x12valid_dataset_size\x18\x02 \x01(\x05\"\xc4\x01\n\x0c\x44\x61tasetGraph\x12+\n\x08\x63hildren\x18\x01 \x03(\x0b\x32\x19.mindinsight.DatasetGraph\x12\x32\n\tparameter\x18\x02 \x01(\x0b\x32\x1f.mindinsight.OperationParameter\x12*\n\noperations\x18\x03 \x03(\x0b\x32\x16.mindinsight.Operation\x12\'\n\x07sampler\x18\x04 \x01(\x0b\x32\x16.mindinsight.Operation\"c\n\tOperation\x12\x37\n\x0eoperationParam\x18\x01 \x01(\x0b\x32\x1f.mindinsight.OperationParameter\x12\x0c\n\x04size\x18\x02 \x03(\x05\x12\x0f\n\x07weights\x18\x03 \x03(\x02\"\xde\x04\n\x12OperationParameter\x12;\n\x06mapStr\x18\x01 \x03(\x0b\x32+.mindinsight.OperationParameter.MapStrEntry\x12\x43\n\nmapStrList\x18\x02 \x03(\x0b\x32/.mindinsight.OperationParameter.MapStrListEntry\x12=\n\x07mapBool\x18\x03 \x03(\x0b\x32,.mindinsight.OperationParameter.MapBoolEntry\x12;\n\x06mapInt\x18\x04 \x03(\x0b\x32+.mindinsight.OperationParameter.MapIntEntry\x12\x41\n\tmapDouble\x18\x05 \x03(\x0b\x32..mindinsight.OperationParameter.MapDoubleEntry\x1a-\n\x0bMapStrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aG\n\x0fMapStrListEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.mindinsight.StrList:\x02\x38\x01\x1a.\n\x0cMapBoolEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a-\n\x0bMapIntEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x30\n\x0eMapDoubleEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\"\x1b\n\x07StrList\x12\x10\n\x08strValue\x18\x01 \x03(\tB\x03\xf8\x01\x01')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_LINEAGEEVENT = _descriptor.Descriptor(
  name='LineageEvent',
  full_name='mindinsight.LineageEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='wall_time', full_name='mindinsight.LineageEvent.wall_time', index=0,
      number=1, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='step', full_name='mindinsight.LineageEvent.step', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='version', full_name='mindinsight.LineageEvent.version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='train_lineage', full_name='mindinsight.LineageEvent.train_lineage', index=3,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='evaluation_lineage', full_name='mindinsight.LineageEvent.evaluation_lineage', index=4,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dataset_graph', full_name='mindinsight.LineageEvent.dataset_graph', index=5,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_defined_info', full_name='mindinsight.LineageEvent.user_defined_info', index=6,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='what', full_name='mindinsight.LineageEvent.what',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=43,
  serialized_end=342,
)


_USERDEFINEDINFO_MAPDICTENTRY = _descriptor.Descriptor(
  name='MapDictEntry',
  full_name='mindinsight.UserDefinedInfo.MapDictEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.UserDefinedInfo.MapDictEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.UserDefinedInfo.MapDictEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=661,
  serialized_end=737,
)

_USERDEFINEDINFO_MAPINT32ENTRY = _descriptor.Descriptor(
  name='MapInt32Entry',
  full_name='mindinsight.UserDefinedInfo.MapInt32Entry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.UserDefinedInfo.MapInt32Entry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.UserDefinedInfo.MapInt32Entry.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=739,
  serialized_end=786,
)

_USERDEFINEDINFO_MAPSTRENTRY = _descriptor.Descriptor(
  name='MapStrEntry',
  full_name='mindinsight.UserDefinedInfo.MapStrEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.UserDefinedInfo.MapStrEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.UserDefinedInfo.MapStrEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=788,
  serialized_end=833,
)

_USERDEFINEDINFO_MAPDOUBLEENTRY = _descriptor.Descriptor(
  name='MapDoubleEntry',
  full_name='mindinsight.UserDefinedInfo.MapDoubleEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.UserDefinedInfo.MapDoubleEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.UserDefinedInfo.MapDoubleEntry.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=835,
  serialized_end=883,
)

_USERDEFINEDINFO = _descriptor.Descriptor(
  name='UserDefinedInfo',
  full_name='mindinsight.UserDefinedInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_info', full_name='mindinsight.UserDefinedInfo.user_info', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='map_dict', full_name='mindinsight.UserDefinedInfo.map_dict', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='map_int32', full_name='mindinsight.UserDefinedInfo.map_int32', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='map_str', full_name='mindinsight.UserDefinedInfo.map_str', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='map_double', full_name='mindinsight.UserDefinedInfo.map_double', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_USERDEFINEDINFO_MAPDICTENTRY, _USERDEFINEDINFO_MAPINT32ENTRY, _USERDEFINEDINFO_MAPSTRENTRY, _USERDEFINEDINFO_MAPDOUBLEENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=345,
  serialized_end=883,
)


_TRAINLINEAGE_HYPERPARAMETERS = _descriptor.Descriptor(
  name='HyperParameters',
  full_name='mindinsight.TrainLineage.HyperParameters',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='optimizer', full_name='mindinsight.TrainLineage.HyperParameters.optimizer', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='learning_rate', full_name='mindinsight.TrainLineage.HyperParameters.learning_rate', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='loss_function', full_name='mindinsight.TrainLineage.HyperParameters.loss_function', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='epoch', full_name='mindinsight.TrainLineage.HyperParameters.epoch', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='parallel_mode', full_name='mindinsight.TrainLineage.HyperParameters.parallel_mode', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='device_num', full_name='mindinsight.TrainLineage.HyperParameters.device_num', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='batch_size', full_name='mindinsight.TrainLineage.HyperParameters.batch_size', index=6,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1139,
  serialized_end=1299,
)

_TRAINLINEAGE_TRAINDATASET = _descriptor.Descriptor(
  name='TrainDataset',
  full_name='mindinsight.TrainLineage.TrainDataset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='train_dataset_path', full_name='mindinsight.TrainLineage.TrainDataset.train_dataset_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='train_dataset_size', full_name='mindinsight.TrainLineage.TrainDataset.train_dataset_size', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1301,
  serialized_end=1371,
)

_TRAINLINEAGE_ALGORITHM = _descriptor.Descriptor(
  name='Algorithm',
  full_name='mindinsight.TrainLineage.Algorithm',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='network', full_name='mindinsight.TrainLineage.Algorithm.network', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='loss', full_name='mindinsight.TrainLineage.Algorithm.loss', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1373,
  serialized_end=1415,
)

_TRAINLINEAGE_MODEL = _descriptor.Descriptor(
  name='Model',
  full_name='mindinsight.TrainLineage.Model',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='mindinsight.TrainLineage.Model.path', index=0,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='size', full_name='mindinsight.TrainLineage.Model.size', index=1,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1417,
  serialized_end=1452,
)

_TRAINLINEAGE = _descriptor.Descriptor(
  name='TrainLineage',
  full_name='mindinsight.TrainLineage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hyper_parameters', full_name='mindinsight.TrainLineage.hyper_parameters', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='train_dataset', full_name='mindinsight.TrainLineage.train_dataset', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='algorithm', full_name='mindinsight.TrainLineage.algorithm', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='model', full_name='mindinsight.TrainLineage.model', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_TRAINLINEAGE_HYPERPARAMETERS, _TRAINLINEAGE_TRAINDATASET, _TRAINLINEAGE_ALGORITHM, _TRAINLINEAGE_MODEL, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=886,
  serialized_end=1452,
)


_EVALUATIONLINEAGE_VALIDDATASET = _descriptor.Descriptor(
  name='ValidDataset',
  full_name='mindinsight.EvaluationLineage.ValidDataset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='valid_dataset_path', full_name='mindinsight.EvaluationLineage.ValidDataset.valid_dataset_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='valid_dataset_size', full_name='mindinsight.EvaluationLineage.ValidDataset.valid_dataset_size', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1560,
  serialized_end=1630,
)

_EVALUATIONLINEAGE = _descriptor.Descriptor(
  name='EvaluationLineage',
  full_name='mindinsight.EvaluationLineage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metric', full_name='mindinsight.EvaluationLineage.metric', index=0,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='valid_dataset', full_name='mindinsight.EvaluationLineage.valid_dataset', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_EVALUATIONLINEAGE_VALIDDATASET, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1455,
  serialized_end=1630,
)


_DATASETGRAPH = _descriptor.Descriptor(
  name='DatasetGraph',
  full_name='mindinsight.DatasetGraph',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='children', full_name='mindinsight.DatasetGraph.children', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='parameter', full_name='mindinsight.DatasetGraph.parameter', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='operations', full_name='mindinsight.DatasetGraph.operations', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sampler', full_name='mindinsight.DatasetGraph.sampler', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1633,
  serialized_end=1829,
)


_OPERATION = _descriptor.Descriptor(
  name='Operation',
  full_name='mindinsight.Operation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='operationParam', full_name='mindinsight.Operation.operationParam', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='size', full_name='mindinsight.Operation.size', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='weights', full_name='mindinsight.Operation.weights', index=2,
      number=3, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1831,
  serialized_end=1930,
)


_OPERATIONPARAMETER_MAPSTRENTRY = _descriptor.Descriptor(
  name='MapStrEntry',
  full_name='mindinsight.OperationParameter.MapStrEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.OperationParameter.MapStrEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.OperationParameter.MapStrEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=788,
  serialized_end=833,
)

_OPERATIONPARAMETER_MAPSTRLISTENTRY = _descriptor.Descriptor(
  name='MapStrListEntry',
  full_name='mindinsight.OperationParameter.MapStrListEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.OperationParameter.MapStrListEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.OperationParameter.MapStrListEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2323,
  serialized_end=2394,
)

_OPERATIONPARAMETER_MAPBOOLENTRY = _descriptor.Descriptor(
  name='MapBoolEntry',
  full_name='mindinsight.OperationParameter.MapBoolEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.OperationParameter.MapBoolEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.OperationParameter.MapBoolEntry.value', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2396,
  serialized_end=2442,
)

_OPERATIONPARAMETER_MAPINTENTRY = _descriptor.Descriptor(
  name='MapIntEntry',
  full_name='mindinsight.OperationParameter.MapIntEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.OperationParameter.MapIntEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.OperationParameter.MapIntEntry.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2444,
  serialized_end=2489,
)

_OPERATIONPARAMETER_MAPDOUBLEENTRY = _descriptor.Descriptor(
  name='MapDoubleEntry',
  full_name='mindinsight.OperationParameter.MapDoubleEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.OperationParameter.MapDoubleEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.OperationParameter.MapDoubleEntry.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=835,
  serialized_end=883,
)

_OPERATIONPARAMETER = _descriptor.Descriptor(
  name='OperationParameter',
  full_name='mindinsight.OperationParameter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mapStr', full_name='mindinsight.OperationParameter.mapStr', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mapStrList', full_name='mindinsight.OperationParameter.mapStrList', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mapBool', full_name='mindinsight.OperationParameter.mapBool', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mapInt', full_name='mindinsight.OperationParameter.mapInt', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mapDouble', full_name='mindinsight.OperationParameter.mapDouble', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_OPERATIONPARAMETER_MAPSTRENTRY, _OPERATIONPARAMETER_MAPSTRLISTENTRY, _OPERATIONPARAMETER_MAPBOOLENTRY, _OPERATIONPARAMETER_MAPINTENTRY, _OPERATIONPARAMETER_MAPDOUBLEENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1933,
  serialized_end=2539,
)


_STRLIST = _descriptor.Descriptor(
  name='StrList',
  full_name='mindinsight.StrList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='strValue', full_name='mindinsight.StrList.strValue', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2541,
  serialized_end=2568,
)

_LINEAGEEVENT.fields_by_name['train_lineage'].message_type = _TRAINLINEAGE
_LINEAGEEVENT.fields_by_name['evaluation_lineage'].message_type = _EVALUATIONLINEAGE
_LINEAGEEVENT.fields_by_name['dataset_graph'].message_type = _DATASETGRAPH
_LINEAGEEVENT.fields_by_name['user_defined_info'].message_type = _USERDEFINEDINFO
_LINEAGEEVENT.oneofs_by_name['what'].fields.append(
  _LINEAGEEVENT.fields_by_name['version'])
_LINEAGEEVENT.fields_by_name['version'].containing_oneof = _LINEAGEEVENT.oneofs_by_name['what']
_LINEAGEEVENT.oneofs_by_name['what'].fields.append(
  _LINEAGEEVENT.fields_by_name['train_lineage'])
_LINEAGEEVENT.fields_by_name['train_lineage'].containing_oneof = _LINEAGEEVENT.oneofs_by_name['what']
_LINEAGEEVENT.oneofs_by_name['what'].fields.append(
  _LINEAGEEVENT.fields_by_name['evaluation_lineage'])
_LINEAGEEVENT.fields_by_name['evaluation_lineage'].containing_oneof = _LINEAGEEVENT.oneofs_by_name['what']
_LINEAGEEVENT.oneofs_by_name['what'].fields.append(
  _LINEAGEEVENT.fields_by_name['dataset_graph'])
_LINEAGEEVENT.fields_by_name['dataset_graph'].containing_oneof = _LINEAGEEVENT.oneofs_by_name['what']
_LINEAGEEVENT.oneofs_by_name['what'].fields.append(
  _LINEAGEEVENT.fields_by_name['user_defined_info'])
_LINEAGEEVENT.fields_by_name['user_defined_info'].containing_oneof = _LINEAGEEVENT.oneofs_by_name['what']
_USERDEFINEDINFO_MAPDICTENTRY.fields_by_name['value'].message_type = _USERDEFINEDINFO
_USERDEFINEDINFO_MAPDICTENTRY.containing_type = _USERDEFINEDINFO
_USERDEFINEDINFO_MAPINT32ENTRY.containing_type = _USERDEFINEDINFO
_USERDEFINEDINFO_MAPSTRENTRY.containing_type = _USERDEFINEDINFO
_USERDEFINEDINFO_MAPDOUBLEENTRY.containing_type = _USERDEFINEDINFO
_USERDEFINEDINFO.fields_by_name['user_info'].message_type = _USERDEFINEDINFO
_USERDEFINEDINFO.fields_by_name['map_dict'].message_type = _USERDEFINEDINFO_MAPDICTENTRY
_USERDEFINEDINFO.fields_by_name['map_int32'].message_type = _USERDEFINEDINFO_MAPINT32ENTRY
_USERDEFINEDINFO.fields_by_name['map_str'].message_type = _USERDEFINEDINFO_MAPSTRENTRY
_USERDEFINEDINFO.fields_by_name['map_double'].message_type = _USERDEFINEDINFO_MAPDOUBLEENTRY
_TRAINLINEAGE_HYPERPARAMETERS.containing_type = _TRAINLINEAGE
_TRAINLINEAGE_TRAINDATASET.containing_type = _TRAINLINEAGE
_TRAINLINEAGE_ALGORITHM.containing_type = _TRAINLINEAGE
_TRAINLINEAGE_MODEL.containing_type = _TRAINLINEAGE
_TRAINLINEAGE.fields_by_name['hyper_parameters'].message_type = _TRAINLINEAGE_HYPERPARAMETERS
_TRAINLINEAGE.fields_by_name['train_dataset'].message_type = _TRAINLINEAGE_TRAINDATASET
_TRAINLINEAGE.fields_by_name['algorithm'].message_type = _TRAINLINEAGE_ALGORITHM
_TRAINLINEAGE.fields_by_name['model'].message_type = _TRAINLINEAGE_MODEL
_EVALUATIONLINEAGE_VALIDDATASET.containing_type = _EVALUATIONLINEAGE
_EVALUATIONLINEAGE.fields_by_name['valid_dataset'].message_type = _EVALUATIONLINEAGE_VALIDDATASET
_DATASETGRAPH.fields_by_name['children'].message_type = _DATASETGRAPH
_DATASETGRAPH.fields_by_name['parameter'].message_type = _OPERATIONPARAMETER
_DATASETGRAPH.fields_by_name['operations'].message_type = _OPERATION
_DATASETGRAPH.fields_by_name['sampler'].message_type = _OPERATION
_OPERATION.fields_by_name['operationParam'].message_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER_MAPSTRENTRY.containing_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER_MAPSTRLISTENTRY.fields_by_name['value'].message_type = _STRLIST
_OPERATIONPARAMETER_MAPSTRLISTENTRY.containing_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER_MAPBOOLENTRY.containing_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER_MAPINTENTRY.containing_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER_MAPDOUBLEENTRY.containing_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER.fields_by_name['mapStr'].message_type = _OPERATIONPARAMETER_MAPSTRENTRY
_OPERATIONPARAMETER.fields_by_name['mapStrList'].message_type = _OPERATIONPARAMETER_MAPSTRLISTENTRY
_OPERATIONPARAMETER.fields_by_name['mapBool'].message_type = _OPERATIONPARAMETER_MAPBOOLENTRY
_OPERATIONPARAMETER.fields_by_name['mapInt'].message_type = _OPERATIONPARAMETER_MAPINTENTRY
_OPERATIONPARAMETER.fields_by_name['mapDouble'].message_type = _OPERATIONPARAMETER_MAPDOUBLEENTRY
DESCRIPTOR.message_types_by_name['LineageEvent'] = _LINEAGEEVENT
DESCRIPTOR.message_types_by_name['UserDefinedInfo'] = _USERDEFINEDINFO
DESCRIPTOR.message_types_by_name['TrainLineage'] = _TRAINLINEAGE
DESCRIPTOR.message_types_by_name['EvaluationLineage'] = _EVALUATIONLINEAGE
DESCRIPTOR.message_types_by_name['DatasetGraph'] = _DATASETGRAPH
DESCRIPTOR.message_types_by_name['Operation'] = _OPERATION
DESCRIPTOR.message_types_by_name['OperationParameter'] = _OPERATIONPARAMETER
DESCRIPTOR.message_types_by_name['StrList'] = _STRLIST

LineageEvent = _reflection.GeneratedProtocolMessageType('LineageEvent', (_message.Message,), dict(
  DESCRIPTOR = _LINEAGEEVENT,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.LineageEvent)
  ))
_sym_db.RegisterMessage(LineageEvent)

UserDefinedInfo = _reflection.GeneratedProtocolMessageType('UserDefinedInfo', (_message.Message,), dict(

  MapDictEntry = _reflection.GeneratedProtocolMessageType('MapDictEntry', (_message.Message,), dict(
    DESCRIPTOR = _USERDEFINEDINFO_MAPDICTENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.UserDefinedInfo.MapDictEntry)
    ))
  ,

  MapInt32Entry = _reflection.GeneratedProtocolMessageType('MapInt32Entry', (_message.Message,), dict(
    DESCRIPTOR = _USERDEFINEDINFO_MAPINT32ENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.UserDefinedInfo.MapInt32Entry)
    ))
  ,

  MapStrEntry = _reflection.GeneratedProtocolMessageType('MapStrEntry', (_message.Message,), dict(
    DESCRIPTOR = _USERDEFINEDINFO_MAPSTRENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.UserDefinedInfo.MapStrEntry)
    ))
  ,

  MapDoubleEntry = _reflection.GeneratedProtocolMessageType('MapDoubleEntry', (_message.Message,), dict(
    DESCRIPTOR = _USERDEFINEDINFO_MAPDOUBLEENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.UserDefinedInfo.MapDoubleEntry)
    ))
  ,
  DESCRIPTOR = _USERDEFINEDINFO,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.UserDefinedInfo)
  ))
_sym_db.RegisterMessage(UserDefinedInfo)
_sym_db.RegisterMessage(UserDefinedInfo.MapDictEntry)
_sym_db.RegisterMessage(UserDefinedInfo.MapInt32Entry)
_sym_db.RegisterMessage(UserDefinedInfo.MapStrEntry)
_sym_db.RegisterMessage(UserDefinedInfo.MapDoubleEntry)

TrainLineage = _reflection.GeneratedProtocolMessageType('TrainLineage', (_message.Message,), dict(

  HyperParameters = _reflection.GeneratedProtocolMessageType('HyperParameters', (_message.Message,), dict(
    DESCRIPTOR = _TRAINLINEAGE_HYPERPARAMETERS,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.TrainLineage.HyperParameters)
    ))
  ,

  TrainDataset = _reflection.GeneratedProtocolMessageType('TrainDataset', (_message.Message,), dict(
    DESCRIPTOR = _TRAINLINEAGE_TRAINDATASET,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.TrainLineage.TrainDataset)
    ))
  ,

  Algorithm = _reflection.GeneratedProtocolMessageType('Algorithm', (_message.Message,), dict(
    DESCRIPTOR = _TRAINLINEAGE_ALGORITHM,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.TrainLineage.Algorithm)
    ))
  ,

  Model = _reflection.GeneratedProtocolMessageType('Model', (_message.Message,), dict(
    DESCRIPTOR = _TRAINLINEAGE_MODEL,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.TrainLineage.Model)
    ))
  ,
  DESCRIPTOR = _TRAINLINEAGE,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.TrainLineage)
  ))
_sym_db.RegisterMessage(TrainLineage)
_sym_db.RegisterMessage(TrainLineage.HyperParameters)
_sym_db.RegisterMessage(TrainLineage.TrainDataset)
_sym_db.RegisterMessage(TrainLineage.Algorithm)
_sym_db.RegisterMessage(TrainLineage.Model)

EvaluationLineage = _reflection.GeneratedProtocolMessageType('EvaluationLineage', (_message.Message,), dict(

  ValidDataset = _reflection.GeneratedProtocolMessageType('ValidDataset', (_message.Message,), dict(
    DESCRIPTOR = _EVALUATIONLINEAGE_VALIDDATASET,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.EvaluationLineage.ValidDataset)
    ))
  ,
  DESCRIPTOR = _EVALUATIONLINEAGE,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.EvaluationLineage)
  ))
_sym_db.RegisterMessage(EvaluationLineage)
_sym_db.RegisterMessage(EvaluationLineage.ValidDataset)

DatasetGraph = _reflection.GeneratedProtocolMessageType('DatasetGraph', (_message.Message,), dict(
  DESCRIPTOR = _DATASETGRAPH,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.DatasetGraph)
  ))
_sym_db.RegisterMessage(DatasetGraph)

Operation = _reflection.GeneratedProtocolMessageType('Operation', (_message.Message,), dict(
  DESCRIPTOR = _OPERATION,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.Operation)
  ))
_sym_db.RegisterMessage(Operation)

OperationParameter = _reflection.GeneratedProtocolMessageType('OperationParameter', (_message.Message,), dict(

  MapStrEntry = _reflection.GeneratedProtocolMessageType('MapStrEntry', (_message.Message,), dict(
    DESCRIPTOR = _OPERATIONPARAMETER_MAPSTRENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter.MapStrEntry)
    ))
  ,

  MapStrListEntry = _reflection.GeneratedProtocolMessageType('MapStrListEntry', (_message.Message,), dict(
    DESCRIPTOR = _OPERATIONPARAMETER_MAPSTRLISTENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter.MapStrListEntry)
    ))
  ,

  MapBoolEntry = _reflection.GeneratedProtocolMessageType('MapBoolEntry', (_message.Message,), dict(
    DESCRIPTOR = _OPERATIONPARAMETER_MAPBOOLENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter.MapBoolEntry)
    ))
  ,

  MapIntEntry = _reflection.GeneratedProtocolMessageType('MapIntEntry', (_message.Message,), dict(
    DESCRIPTOR = _OPERATIONPARAMETER_MAPINTENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter.MapIntEntry)
    ))
  ,

  MapDoubleEntry = _reflection.GeneratedProtocolMessageType('MapDoubleEntry', (_message.Message,), dict(
    DESCRIPTOR = _OPERATIONPARAMETER_MAPDOUBLEENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter.MapDoubleEntry)
    ))
  ,
  DESCRIPTOR = _OPERATIONPARAMETER,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter)
  ))
_sym_db.RegisterMessage(OperationParameter)
_sym_db.RegisterMessage(OperationParameter.MapStrEntry)
_sym_db.RegisterMessage(OperationParameter.MapStrListEntry)
_sym_db.RegisterMessage(OperationParameter.MapBoolEntry)
_sym_db.RegisterMessage(OperationParameter.MapIntEntry)
_sym_db.RegisterMessage(OperationParameter.MapDoubleEntry)

StrList = _reflection.GeneratedProtocolMessageType('StrList', (_message.Message,), dict(
  DESCRIPTOR = _STRLIST,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.StrList)
  ))
_sym_db.RegisterMessage(StrList)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\370\001\001'))
_USERDEFINEDINFO_MAPDICTENTRY.has_options = True
_USERDEFINEDINFO_MAPDICTENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_USERDEFINEDINFO_MAPINT32ENTRY.has_options = True
_USERDEFINEDINFO_MAPINT32ENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_USERDEFINEDINFO_MAPSTRENTRY.has_options = True
_USERDEFINEDINFO_MAPSTRENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_USERDEFINEDINFO_MAPDOUBLEENTRY.has_options = True
_USERDEFINEDINFO_MAPDOUBLEENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_OPERATIONPARAMETER_MAPSTRENTRY.has_options = True
_OPERATIONPARAMETER_MAPSTRENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_OPERATIONPARAMETER_MAPSTRLISTENTRY.has_options = True
_OPERATIONPARAMETER_MAPSTRLISTENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_OPERATIONPARAMETER_MAPBOOLENTRY.has_options = True
_OPERATIONPARAMETER_MAPBOOLENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_OPERATIONPARAMETER_MAPINTENTRY.has_options = True
_OPERATIONPARAMETER_MAPINTENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_OPERATIONPARAMETER_MAPDOUBLEENTRY.has_options = True
_OPERATIONPARAMETER_MAPDOUBLEENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)



# Copyright 2020 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Utils for optimizer."""
import string

import numpy as np

_DEFAULT_HISTOGRAM_BINS = 5


def calc_histogram(np_value: np.ndarray, bins=_DEFAULT_HISTOGRAM_BINS):
    """
    Calculates histogram.

    This is a simple wrapper around the error-prone np.histogram() to improve robustness.
    """
    ma_value = np.ma.masked_invalid(np_value)

    valid_cnt = ma_value.count()
    if not valid_cnt:
        max_val = 0
        min_val = 0
    else:
        # Note that max of a masked array with dtype np.float16 returns inf (numpy issue#15077).
        if np.issubdtype(np_value.dtype, np.floating):
            max_val = ma_value.max(fill_value=np.NINF)
            min_val = ma_value.min(fill_value=np.PINF)
        else:
            max_val = ma_value.max()
            min_val = ma_value.min()

    range_left = min_val
    range_right = max_val

    default_half_range = 0.5
    if range_left >= range_right:
        range_left -= default_half_range
        range_right += default_half_range

    with np.errstate(invalid='ignore'):
        # if don't ignore state above, when np.nan exists,
        # it will occur RuntimeWarning: invalid value encountered in less_equal
        counts, edges = np.histogram(np_value, bins=bins, range=(range_left, range_right))

    histogram_bins = [None] * len(counts)
    for ind, count in enumerate(counts):
        histogram_bins[ind] = [float(edges[ind]), float(edges[ind + 1] - edges[ind]), float(count)]

    return histogram_bins


def is_simple_numpy_number(dtype):
    """Verify if it is simple number."""
    if np.issubdtype(dtype, np.integer):
        return True

    if np.issubdtype(dtype, np.floating):
        return True

    return False


def get_nested_message(info: dict, out_err_msg=""):
    """Get error message from the error dict generated by schema validation."""
    if not isinstance(info, dict):
        if isinstance(info, list):
            info = info[0]
        return f'Error in {out_err_msg}: {info}'
    for key in info:
        if isinstance(key, str) and key != '_schema':
            if out_err_msg:
                out_err_msg = f'{out_err_msg}.{key}'
            else:
                out_err_msg = key
        return get_nested_message(info[key], out_err_msg)


def is_number(uchar):
    """If it is a number, return True."""
    if uchar in string.digits:
        return True
    return False


def is_alphabet(uchar):
    """If it is an alphabet, return True."""
    if uchar in string.ascii_letters:
        return True
    return False


def is_allowed_symbols(uchar):
    """If it is an allowed symbol, return True."""
    if uchar in ['_']:
        return True
    return False


def is_param_name_valid(param_name: str):
    """If parameter name only contains underscore, number or alphabet, return True."""
    for uchar in param_name:
        if not is_number(uchar) and not is_alphabet(uchar) and not is_allowed_symbols(uchar):
            return False
    return True



# Copyright 2020 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Optimizer API module."""
import json
from flask import Blueprint, jsonify, request

from mindinsight.conf import settings
from mindinsight.datavisual.data_transform.data_manager import DATA_MANAGER
from mindinsight.lineagemgr.model import get_lineage_table
from mindinsight.optimizer.common.enums import ReasonCode
from mindinsight.optimizer.common.exceptions import SamplesNotEnoughError, CorrelationNanError
from mindinsight.optimizer.utils.importances import calc_hyper_param_importance
from mindinsight.optimizer.utils.utils import calc_histogram
from mindinsight.utils.exceptions import ParamValueError

BLUEPRINT = Blueprint("optimizer", __name__, url_prefix=settings.URL_PATH_PREFIX+settings.API_PREFIX)


@BLUEPRINT.route("/optimizer/targets/search", methods=["POST"])
def get_optimize_targets():
    """Get optimize targets."""
    search_condition = request.stream.read()
    try:
        search_condition = json.loads(search_condition if search_condition else "{}")
    except Exception:
        raise ParamValueError("Json data parse failed.")

    response = _get_optimize_targets(DATA_MANAGER, search_condition)
    return jsonify(response)


def _get_optimize_targets(data_manager, search_condition=None):
    """Get optimize targets."""
    table = get_lineage_table(data_manager=data_manager, search_condition=search_condition)

    target_summaries = []
    for target in table.target_names:
        hyper_parameters = []
        for hyper_param in table.hyper_param_names:
            param_info = {"name": hyper_param}
            try:
                importance = calc_hyper_param_importance(table.dataframe_data, hyper_param, target)
                param_info.update({"importance": importance})
            except SamplesNotEnoughError:
                param_info.update({"importance": 0})
                param_info.update({"reason_code": ReasonCode.SAMPLES_NOT_ENOUGH.value})
            except CorrelationNanError:
                param_info.update({"importance": 0})
                param_info.update({"reason_code": ReasonCode.CORRELATION_NAN.value})
            hyper_parameters.append(param_info)

        # Sort `hyper_parameters` in descending order of `importance` and ascending order of `name`.
        # If the automatically collected parameters and user-defined parameters have the same importance,
        # the user-defined parameters will be ranked behind.
        hyper_parameters.sort(key=lambda hyper_param: (-hyper_param.get("importance"),
                                                       hyper_param.get("name").startswith('['),
                                                       hyper_param.get("name")))

        target_summary = {
            "name": target,
            "buckets": calc_histogram(table.get_column(target)),
            "hyper_parameters": hyper_parameters,
            "data": table.get_column_values(target)
        }
        target_summaries.append(target_summary)

    target_summaries.sort(key=lambda summary: summary.get("name"))

    hyper_params_metadata = [{
        "name": hyper_param,
        "data": table.get_column_values(hyper_param)
    } for hyper_param in table.hyper_param_names]

    result = {
        "metadata": {
            "train_ids": table.train_ids,
            "possible_hyper_parameters": hyper_params_metadata,
            "unrecognized_params": table.drop_column_info
        },
        "targets": target_summaries
    }

    return result


def init_module(app):
    """
    Init module entry.

    Args:
        app: the application obj.

    """
    app.register_blueprint(BLUEPRINT)



# Copyright 2019-2021 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Lineage error code and messages."""
from enum import Enum, unique
from mindinsight.utils.constant import LineageMgrErrors as LineageErrorCodes


_PARAM_ERROR_MASK = 0b00001 << 7
_MINDSPORE_COLLECTOR_ERROR = 0b00011 << 7
_MODEL_LINEAGE_API_ERROR_MASK = 0b00100 << 7
_DATASET_COLLECTOR_ERROR_MASK = 0b00101 << 7
_DATASET_LINEAGE_ERROR_MASK = 0b00110 << 7
_SUMMARY_ANALYZE_ERROR_MASK = 0b00111 << 7
_QUERIER_ERROR_MASK = 0b01000 << 7
_FILE_HANDLER_MASK = 0b01001 << 7

@unique
class LineageErrors(LineageErrorCodes):
    """Lineage error codes."""
    PARAM_TYPE_ERROR = 0 | _PARAM_ERROR_MASK
    PARAM_VALUE_ERROR = 1 | _PARAM_ERROR_MASK
    PARAM_MISSING_ERROR = 2 | _PARAM_ERROR_MASK
    PARAM_SUMMARY_RECORD_ERROR = 3 | _PARAM_ERROR_MASK
    PARAM_RAISE_EXCEPTION_ERROR = 4 | _PARAM_ERROR_MASK

    # MindSpore Collector error codes.
    PARAM_RUN_CONTEXT_ERROR = 0 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_OPTIMIZER_ERROR = 1 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_LOSS_FN_ERROR = 2 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_TRAIN_NETWORK_ERROR = 3 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_DATASET_ERROR = 4 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_EPOCH_NUM_ERROR = 5 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_BATCH_NUM_ERROR = 6 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_TRAIN_PARALLEL_ERROR = 7 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_DEVICE_NUMBER_ERROR = 8 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_FILE_PATH_ERROR = 9 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_DATASET_SIZE_ERROR = 10 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_LEARNING_RATE_ERROR = 11 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_EVAL_METRICS_ERROR = 12 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_BATCH_SIZE_ERROR = 13 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_NET_OUTPUTS_ERROR = 14 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_CALLBACK_LIST_ERROR = 15 | _MINDSPORE_COLLECTOR_ERROR
    LINEAGE_GET_MODEL_FILE_ERROR = 16 | _MINDSPORE_COLLECTOR_ERROR
    LOG_LINEAGE_INFO_ERROR = 17 | _MINDSPORE_COLLECTOR_ERROR
    PARAM_STEP_NUM_ERROR = 18 | _MINDSPORE_COLLECTOR_ERROR

    # Model lineage error codes.
    LINEAGE_PARAM_METRIC_ERROR = 1 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_LOSS_FUNCTION_ERROR = 4 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_TRAIN_DATASET_PATH_ERROR = 5 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_TRAIN_DATASET_COUNT_ERROR = 6 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_TEST_DATASET_PATH_ERROR = 7 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_TEST_DATASET_COUNT_ERROR = 8 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_NETWORK_ERROR = 9 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_OPTIMIZER_ERROR = 10 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_LEARNING_RATE_ERROR = 11 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_EPOCH_ERROR = 12 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_BATCH_SIZE_ERROR = 13 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_NOT_SUPPORT_ERROR = 14 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_LOSS_ERROR = 15 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_MODEL_SIZE_ERROR = 16 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_SUMMARY_DIR_ERROR = 17 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_SORTED_NAME_ERROR = 18 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_SORTED_TYPE_ERROR = 19 | _MODEL_LINEAGE_API_ERROR_MASK

    LINEAGE_DIR_NOT_EXIST_ERROR = 20 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_SUMMARY_DATA_ERROR = 21 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_FILE_NOT_FOUND_ERROR = 22 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_SEARCH_CONDITION_PARAM_ERROR = 24 | _MODEL_LINEAGE_API_ERROR_MASK

    LINEAGE_PARAM_LINEAGE_TYPE_ERROR = 25 | _MODEL_LINEAGE_API_ERROR_MASK
    LINEAGE_PARAM_DEVICE_NUM_ERROR = 26 | _MODEL_LINEAGE_API_ERROR_MASK

    # Dataset lineage error codes.
    LINEAGE_PARAM_DATASET_MARK_ERROR = 0 | _DATASET_LINEAGE_ERROR_MASK

    SUMMARY_ANALYZE_ERROR = 0 | _SUMMARY_ANALYZE_ERROR_MASK
    SUMMARY_VERIFICATION_ERROR = 1 | _SUMMARY_ANALYZE_ERROR_MASK

    # Querier error codes.
    EVENT_NOT_EXIST_ERROR = 0 | _QUERIER_ERROR_MASK
    QUERIER_PARAM_ERROR = 1 | _QUERIER_ERROR_MASK
    SUMMARY_PARSE_FAIL_ERROR = 2 | _QUERIER_ERROR_MASK
    EVENT_FIELD_NOT_EXIST_ERROR = 4 | _QUERIER_ERROR_MASK

    # File_handler read error codes
    LINEAGE_FILE_HANDLER_READ_ERROR = 0 | _FILE_HANDLER_MASK

@unique
class LineageErrorMsg(Enum):
    """Lineage error messages."""
    PARAM_TYPE_ERROR = "TypeError. {}"
    PARAM_VALUE_ERROR = "ValueError. {}"
    PARAM_MISSING_ERROR = "MissingError. {}"
    PARAM_LIMIT_ERROR = "Invalid input limit. 0 < limit <= 100"
    PARAM_OFFSET_ERROR = "Invalid input offset. 0 <= offset <= 100000"
    PARAM_SUMMARY_RECORD_ERROR = "Invalid value for summary_record. It should be an instance of " \
                                 "mindspore.train.summary.SummaryRecord"
    PARAM_RAISE_EXCEPTION_ERROR = "Invalid value for raise_exception. It should be True or False."
    # Lineage error messages.
    LINEAGE_SUMMARY_DATA_ERROR = "Query summary data error: {}"
    LINEAGE_FILE_NOT_FOUND_ERROR = "File not found error: {}"
    LINEAGE_DIR_NOT_EXIST_ERROR = "Dir not exist error: {}"
    LINEAGE_SEARCH_CONDITION_PARAM_ERROR = "Search_condition param error: {}"

    # MindSpore Collector error messages.
    PARAM_RUN_CONTEXT_ERROR = "The parameter run_context is invalid. It should be an instance of " \
                              "mindspore.train.callback.RunContext. {}"

    PARAM_OPTIMIZER_ERROR = "The parameter optimizer is invalid. It should be an instance of " \
                            "mindspore.nn.optim.optimizer.Optimizer."

    PARAM_LOSS_FN_ERROR = "The parameter loss_fn is invalid. It should be a Function."

    PARAM_NET_OUTPUTS_ERROR = "The parameter net_outputs is invalid. It should be a Tensor."

    PARAM_TRAIN_NETWORK_ERROR = "The parameter train_network is invalid. It should be an instance of " \
                                "mindspore.nn.cell.Cell."

    PARAM_EPOCH_NUM_ERROR = "The parameter epoch is invalid. It should be a positive integer."

    PARAM_STEP_NUM_ERROR = "The parameter step_num is invalid. It should be a positive integer."

    PARAM_BATCH_NUM_ERROR = "The parameter batch_num is invalid. It should be a non-negative integer."

    PARAM_TRAIN_PARALLEL_ERROR = "The parameter parallel_mode is invalid. It should be an integer" \
                                 "between 0 and 4."

    PARAM_DEVICE_NUMBER_ERROR = "The parameter device_number is invalid. It should be a positive integer."

    PARAM_LEARNING_RATE_ERROR = "The parameter learning_rate is invalid. It should be a float number or " \
                                "an instance of mindspore.common.tensor.Tensor."

    PARAM_EVAL_METRICS_ERROR = "The parameter metrics is invalid. It should be a dictionary."

    PARAM_BATCH_SIZE_ERROR = "The parameter batch_size is invalid. It should be a non-negative integer."

    PARAM_DEVICE_NUM_ERROR = "The parameter device_num is invalid. It should be a non-negative integer."

    PARAM_CALLBACK_LIST_ERROR = "The parameter list_callback is invalid. It should be an instance of " \
                                "mindspore.train.callback._ListCallback."

    LINEAGE_GET_MODEL_FILE_ERROR = "Error when get model file size. {}"

    LINEAGE_METRIC_ERROR = "The parameter {} is invalid. " \
                           "It should be a dict and the value should be a float or a integer"

    LINEAGE_COMPARE_OPERATION_ERROR = "The schema error and compare operation should be" \
                                      " 'eq', 'lt', 'gt', 'ge', 'le', 'in'."

    LINEAGE_PARAM_SUMMARY_DIR_ERROR = "The parameter summary_dir is invalid. It should be a dict and the value " \
                                      "should be a string."

    LINEAGE_TRAIN_DATASET_PATH_ERROR = "The parameter train_dataset_path is invalid." \
                                       " It should be a dict and the value should be a string."

    LINEAGE_TRAIN_DATASET_COUNT_ERROR = "The parameter train_dataset_count is invalid. It should be a dict " \
                                        "and the value should be a integer between 0 and pow(2, 63) -1."

    LINEAGE_TEST_DATASET_PATH_ERROR = "The parameter test_dataset_path is invalid. " \
                                      "It should be a dict and the value should be a string."

    LINEAGE_TEST_DATASET_COUNT_ERROR = "The parameter test_dataset_count is invalid. It should be a dict " \
                                       "and the value should be a integer between 0 and pow(2, 63) -1."

    LINEAGE_NETWORK_ERROR = "The parameter network is invalid. It should be a dict and the value should be a string."

    LINEAGE_OPTIMIZER_ERROR = "The parameter optimizer is invalid. " \
                              "It should be a dict and the value should be a string."

    LINEAGE_LOSS_FUNCTION_ERROR = "The parameter loss_function is invalid. " \
                                  "It should be a dict and the value should be a string."

    LINEAGE_LOSS_ERROR = "The parameter loss is invalid. " \
                         "It should be a float."

    LINEAGE_MODEL_SIZE_ERROR = "The parameter model_size is invalid. " \
                               "It should be an integer between 0 and pow(2, 63) -1."

    LINEAGE_LEARNING_RATE_ERROR = "The parameter learning_rate is invalid. " \
                                  "It should be a dict and the value should be a float or a integer."

    LINEAGE_PARAM_SORTED_NAME_ERROR = "The parameter sorted_name is invalid. " \
                                      "It should be a string."

    LINEAGE_PARAM_SORTED_TYPE_ERROR = "The parameter sorted_type is invalid. " \
                                      "It should be a string."

    LINEAGE_PARAM_LINEAGE_TYPE_ERROR = "The parameter lineage_type is invalid. " \
                                       "It should be 'dataset' or 'model'."

    LINEAGE_PARAM_DATASET_MARK_ERROR = "The parameter dataset_mark is invalid. " \
                                       "It should be a string."

    SUMMARY_ANALYZE_ERROR = "Failed to analyze summary log. {}"
    SUMMARY_VERIFICATION_ERROR = "Verification failed in summary analysis. {}"

    # Querier error codes.
    EVENT_NOT_EXIST_ERROR = "Train and evaluation event not exist in summary log."
    QUERIER_PARAM_ERROR = "Querier param <{}> invalid. {}"
    SUMMARY_PARSE_FAIL_ERROR = "All summary logs parsing failed."
    EVENT_FIELD_NOT_EXIST_ERROR = 'Event field <{}> not exist.'

    LOG_LINEAGE_INFO_ERROR = "Fail to write lineage information into log file. {}"

    LINEAGE_FILE_HANDLER_READ_ERROR = "Fail to read in cache. {}"



# Copyright 2020 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""NetworkUtility module."""
from importlib import import_module


from mindinsight.wizard.common.exceptions import CommandError


def find_network_maker_names():
    return ['lenet', 'alexnet', 'resnet50']


def load_network_maker(network_name):
    module = import_module(f'mindinsight.wizard.network.{network_name.lower()}')
    return module.Network()


def load_dataset_maker(dataset_name, **kwargs):
    module = import_module(f'mindinsight.wizard.dataset.{dataset_name.lower()}')
    return module.Dataset(**kwargs)


def process_prompt_choice(value, prompt_type):
    """Convert command value to business value."""
    if value is not None:
        idx = prompt_type(value)
        return idx
    raise CommandError("The choice is not exist, please choice again.")



# Copyright 2019 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""File handler for file operations."""
from mindinsight.utils.exceptions import PathNotExistError
from mindinsight.datavisual.common.log import logger
from mindinsight.datavisual.utils.tools import to_str
from mindinsight.datavisual.data_access.local_file_system import LocalFileSystem

_DEFAULT_BUFFER_SIZE = 24 * 1024 * 1024

# _FILE_SYSTEMS, key: FileProtocolHead, value: FileSystem
_FILE_SYSTEMS = dict()
_FILE_SYSTEMS[""] = LocalFileSystem()


class FileHandler:
    """File handler."""

    def __init__(self, file_path, mode='rb'):
        """
        Init FileHandler.

        Args:
            file_path (str): File path.
            mode (Literal['r', 'rb', 'br', 'w', 'wb', 'bw']): It must be
                in ['r', 'rb', 'br', 'w', 'wb', 'bw'].
        """
        logger.debug("The __init__ method enter, param: file_path=%s"
                     "mode=%s", file_path, mode)

        if mode not in ('r', 'rb', 'br', 'w', 'wb', 'bw'):
            raise ValueError("mode %s is not supported by FileHandler." % mode)

        self._file_path = to_str(file_path)
        self._file_system = self.get_file_system(self._file_path)
        self._buff_chunk_size = _DEFAULT_BUFFER_SIZE
        self._buff = None
        self._buff_offset = 0
        self._offset = 0
        self._binary_mode = 'b' in mode

    @staticmethod
    def get_file_system(path):
        """
        Get file system object from path.

        Args:
            path (str): Directory path or file path.

        Returns:
            BaseFileSystem, a file system object.
        """
        path = to_str(path)
        prefix_index = path.find("://")
        prefix = path[:prefix_index] if prefix_index >= 0 else ""
        file_system = _FILE_SYSTEMS.get(prefix, None)

        if file_system is None:
            raise ValueError("No filesystem can be found for prefix %s" % prefix)
        return file_system

    @staticmethod
    def walk(node, forward=True, onerror=None):
        """
        Traverse path for directory and file tree.

        Read from the buffer first.If there is not enough data in the buffer,
        data will be read from the file system.

        Args:
            node (str): Current path.
            forward (bool): If True, it will return the sub-directories and files in the top-level
                directory first and then iterate the files in the sub-directories. Default: True.
            onerror (Optional[Callable]): If None, it indicates that errors during file traversal
                will be ignored. Default: None.

        Yields:
            Tuple, (node, sub_dirs, files).

        """
        logger.debug("The walk method enter, param: node=%s, "
                     "forward=%s, onerror=%s.", node, forward, type(onerror))

        file_system = FileHandler.get_file_system(node)
        node = to_str(node)
        dirs = []

        try:
            dirs = file_system.list_dir(node)
        except PathNotExistError as err:
            if onerror:
                onerror(err)
            else:
                logger.warning("Get dir list error, dir_path=%s error=%s.", node, str(err))
                return

        sub_dirs, files = [], []
        for item in dirs:
            full_path = file_system.join(node, to_str(item))
            if file_system.is_dir(full_path):
                sub_dirs.append(item)
            else:
                files.append(item)

        result = (node, sub_dirs, files)

        if forward:
            logger.debug("The walk method return, pre result=%s.", result)
            yield result

        for subdir in sub_dirs:
            joined_subdir = file_system.join(node, to_str(subdir))
            for sub_results in FileHandler.walk(joined_subdir, forward, onerror):
                yield sub_results

        if not forward:
            logger.debug("The walk method return, post result=%s.", result)
            yield result

    def read(self, size=None):
        """
        Read bytes from buffer or file by size.

        Args:
            size (Union[None, int]): Number of bytes to read, If set None, read the whole file. Default: None.

        Returns:
            str, a certain number of bytes.
        """
        if size is None:
            result = self._file_system.read(self._file_path, self._binary_mode)
            self._offset = len(result)
            return result

        result = None
        if self._buff and len(self._buff) > self._buff_offset:
            read_offset = self._buff_offset + size if size is not None else len(self._buff)
            result = self._read_buffer_by_offset(read_offset)
            if size is not None:
                if len(result) == size:
                    return result
                size -= len(result)

        read_size = max(self._buff_chunk_size, size) if size is not None else None
        self._buff = self._file_system.read(self._file_path, self._binary_mode,
                                            read_size, self._offset)
        self._buff_offset = 0

        read_offset = size if size is not None else len(self._buff)
        chunk = self._read_buffer_by_offset(read_offset)

        result = result + chunk if result else chunk

        return result

    def _read_buffer_by_offset(self, new_buff_offset):
        """
        Read buffer by offset.

        Args:
            new_buff_offset (int): Ending offset to read.

        Returns:
            str, bytes from old offset to new offset.

        """
        old_buff_offset = self._buff_offset
        read_size = min(len(self._buff), new_buff_offset) - old_buff_offset
        self._offset += read_size
        self._buff_offset += read_size
        return self._buff[old_buff_offset:old_buff_offset + read_size]

    def reset_offset(self, offset):
        """
        Reset offset and buff_offset, clean buff.

        Args:
            offset (int): Offset.

        """
        self._offset = offset
        self._buff = None
        self._buff_offset = 0

    @staticmethod
    def list_dir(path):
        """
        List directories by path.

        Args:
            path (str): Directory path or file path.

        Returns:
            list[str], directories.
        """
        file_system = FileHandler.get_file_system(path)
        return file_system.list_dir(path)

    @staticmethod
    def is_dir(path):
        """
        Determine if it is a directory.

        Args:
            path (str): Directory path or file path.

        Returns:
            bool, if it is a directory path, return True.
        """
        file_system = FileHandler.get_file_system(path)
        return file_system.is_dir(path)

    @staticmethod
    def is_file(path):
        """
        Determine if it is a file.

        Args:
            path (str): Directory path or file path.

        Returns:
            bool, if it is a file path, return True.
        """
        file_system = FileHandler.get_file_system(path)
        return file_system.is_file(path)

    @staticmethod
    def exists(path):
        """
        Determine if it exists.

        Args:
            path (str): Directory path or file path.

        Returns:
            bool, if it exists, return True.
        """
        file_system = FileHandler.get_file_system(path)
        return file_system.exists(path)

    @staticmethod
    def file_stat(file_path):
        """
        Get file stat information.

        Args:
            file_path (str): File path.

        Returns:
            Nametuple, the (size, mtime) of file.
        """
        file_system = FileHandler.get_file_system(file_path)
        return file_system.file_stat(file_path)

    @staticmethod
    def join(path, *paths):
        """
        Join paths.

        Args:
            path (str): Directory path.
            paths (str): Path or paths.

        Returns:
            str, the joined path.
        """
        file_system = FileHandler.get_file_system(path)
        return file_system.join(path, *paths)

    @property
    def offset(self):
        """Get offset."""
        return self._offset

    @property
    def file_path(self):
        """Get file path."""
        return self._file_path



# Copyright 2020-2021 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Tensor data container."""
import numpy as np

from mindinsight.datavisual.data_transform.histogram import Histogram, Bucket
from mindinsight.datavisual.utils.utils import calc_histogram_bins
from mindinsight.datavisual.common.exceptions import TensorTooLargeError
from mindinsight.utils.exceptions import ParamValueError
from mindinsight.utils.tensor import TensorUtils

MAX_TENSOR_COUNT = 10000000
TENSOR_TOO_LARGE_ERROR = TensorTooLargeError("").error_code


def calc_original_buckets(np_value, stats):
    """
    Calculate buckets from tensor data.

    Args:
        np_value (numpy.ndarray): An numpy.ndarray of tensor data.
        stats (Statistics): An instance of Statistics about tensor data.

    Returns:
        list, a list of bucket about tensor data.

    Raises:
        ParamValueError, If np_value or stats is None.
    """
    if np_value is None or stats is None:
        raise ParamValueError("Invalid input. np_value or stats is None.")
    valid_count = stats.count - stats.nan_count - stats.neg_inf_count - stats.pos_inf_count
    if not valid_count:
        return []

    bins = calc_histogram_bins(valid_count)
    first_edge, last_edge = stats.min, stats.max

    if not first_edge < last_edge:
        first_edge -= 0.5
        last_edge += 0.5

    bins = np.linspace(first_edge, last_edge, bins + 1, dtype=np_value.dtype)
    hists, edges = np.histogram(np_value, bins=bins)

    buckets = []
    for hist, edge1, edge2 in zip(hists, edges, edges[1:]):
        bucket = Bucket(edge1, edge2 - edge1, hist)
        buckets.append(bucket)

    return buckets


class TensorContainer:
    """
    Tensor data container.

    Args:
        tensor_message (Summary.TensorProto): Tensor message in summary file.
    """

    def __init__(self, tensor_message):
        # Original dims can not be pickled to transfer to other process, so tuple is used.
        self._dims = tuple(tensor_message.dims)
        self._data_type = tensor_message.data_type
        self._empty = len(tensor_message.float_data) == 0
        self._np_array = self.get_ndarray(tensor_message.float_data) if not self._empty else np.array([])
        self._error_code = None
        if self._np_array.size > MAX_TENSOR_COUNT:
            self._error_code = TENSOR_TOO_LARGE_ERROR
            self._np_array = np.array([])
        self._stats = TensorUtils.get_statistics_from_tensor(self._np_array)
        original_buckets = calc_original_buckets(self._np_array, self._stats)
        self._count = sum(bucket.count for bucket in original_buckets)
        self._max = self._stats.max
        self._min = self._stats.min
        self._histogram = Histogram(tuple(original_buckets), self._max, self._min, self._count)

    @property
    def empty(self):
        """Get size of tensor."""
        return self._empty

    @property
    def size(self):
        """Get size of tensor."""
        return self._np_array.size

    @property
    def error_code(self):
        """Get size of tensor."""
        return self._error_code

    @property
    def dims(self):
        """Get dims of tensor."""
        return self._dims

    @property
    def data_type(self):
        """Get data type of tensor."""
        return self._data_type

    @property
    def tensor_value(self):
        """Get ndarray of tensor."""
        return self._np_array

    @property
    def max(self):
        """Get max value of tensor."""
        return self._max

    @property
    def min(self):
        """Get min value of tensor."""
        return self._min

    @property
    def stats(self):
        """Get statistics data of tensor."""
        return self._stats

    @property
    def count(self):
        """Get count value of tensor."""
        return self._count

    @property
    def histogram(self):
        """Get histogram data."""
        return self._histogram

    def buckets(self):
        """Get histogram buckets."""
        if self._histogram is None:
            return None
        return self._histogram.buckets()

    def get_ndarray(self, tensor):
        """
        Get ndarray of tensor.

        Args:
            tensor (mindinsight_anf_ir.proto.DataType): tensor data.

        Returns:
            numpy.ndarray, ndarray of tensor.
        """
        return np.array(tuple(tensor)).reshape(self.dims)



# Copyright 2020 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Utils."""
import math
from mindinsight.datavisual.common.log import logger


def calc_histogram_bins(count):
    """
    Calculates experience-based optimal bins number for histogram.

    To suppress re-sample bias, there should be enough number in each bin. So we calc bin numbers according to
    count. For very small count(1 - 10), we assign carefully chosen number. For large count, we tried to make
    sure there are 9-10 numbers in each bucket on average. Too many bins will also distract users, so we set max
    number of bins to 30.

    Args:
        count (int): Valid number count for the tensor.

    Returns:
        int, number of histogram bins.
    """
    number_per_bucket = 10
    max_bins = 30

    if not count:
        return 1
    if count <= 5:
        return 2
    if count <= 10:
        return 3
    if count <= 280:
        # note that math.ceil(281/10) + 1 equals 30
        return math.ceil(count / number_per_bucket) + 1

    return max_bins


def contains_null_byte(**kwargs):
    """
    Check if arg contains null byte.

    Args:
        kwargs (Any): Check if arg contains null byte.

    Returns:
        bool, indicates if any arg contains null byte.
    """
    for key, value in kwargs.items():
        if not isinstance(value, str):
            continue
        if '\x00' in value:
            logger.warning('%s contains null byte \\x00.', key)
            return True

    return False



# Copyright 2020-2022 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Define the util."""

MS_VERSION = '1.0.x'


def version_match(ms_version, mi_version):
    """
    Judge if the version of Mindinsight and Mindspore is matched.
    0 means the version is totally matched;
    1 means the major version is matched but the minor version is mismatch;
    2 means the version is totally mismatch.
    """
    if not ms_version:
        ms_version = MS_VERSION
    mi_major, mi_minor = mi_version.split('.')[:2]
    ms_major, ms_minor = ms_version.split('.')[:2]
    mi_major_num = int(mi_major)
    mi_minor_num = int(mi_minor)
    ms_major_num = int(ms_major)
    ms_minor_num = int(ms_minor)
    if ms_major_num == mi_major_num and ms_minor_num == mi_minor_num:
        return 0
    if ms_major_num == mi_major_num and ms_minor_num > mi_minor_num:
        return 1
    return 2



# Copyright 2020-2021 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Modelarts notebook Registry."""

class Registry:
    """
    Registry to map strings to classes.

    Args:
        name (str): Registry name.
    """

    def __init__(self, name):
        self.name = name
        self._modules = dict()

    @property
    def modules(self):
        return self._modules

    def get(self, key):
        return self._modules.get(key, None)

    def _register_module(self, module, name=None, force=False):
        """
        Register a module.
        """
        if not isinstance(module, type):
            raise ValueError(f'Not supported type: {type(module)} [type]')

        if name is None:
            name = module.__name__
        if not isinstance(name, str):
            raise ValueError(f'Not supported type: {type(name)} [str]')

        if not (force or name not in self._modules):
            raise ValueError(f'Name({name}) is already registered in registry({self.name})')

        self._modules[name] = module

    def register_module(self, name=None, force=False, module=None):
        """
        Register a module.

        Args:
            name (str): The name to be registered. If `None`, the module name
                will be used.
            force (bool): Whether to override an existing module with the same
                name. Default: False.
            module (type): Module to be registered.
        """
        if module is not None:
            self._register_module(
                module=module, name=name, force=force)
            return module

        # Used as a decorator: @xxx.register_module()
        def _register(cls):
            self._register_module(module=cls, name=name, force=force)
            return cls

        return _register



# Copyright 2019 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Mindinsight init module."""

from mindinsight._version import VERSION
from mindinsight.modelarts import notebook

__version__ = VERSION
__version_info__ = tuple(VERSION.split('.'))

__all__ = [
    '__version__',
    '__version_info__'
]


def load_ipython_extension(ipython):
    """
    IPython API entry point.

    Only intended to be called by the IPython runtime.

    See:
       The IPython extension guide.
    """
    notebook.notebook_load_ipython_extension(ipython)



# Copyright 2020 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Parse summary file."""

import argparse
import os
import datetime

from mindinsight.utils.command import BaseCommand
from mindinsight.utils.exceptions import UnknownError
from mindinsight.datavisual.common.log import parse_summary_logger
from mindinsight.datavisual.data_access.file_handler import FileHandler
from mindinsight.datavisual.data_transform.ms_data_loader import _SummaryParser
from mindinsight.datavisual.data_transform.summary_parser.event_parser import EventParser


class DirAction(argparse.Action):
    """File directory action class definition."""

    @staticmethod
    def check_path(file_path):
        """
        Check argument for file path.

        Args:
            file_path (str): File path.
        """
        if file_path.startswith('~'):
            file_path = os.path.realpath(os.path.expanduser(file_path))

        if not file_path.startswith('/'):
            file_path = os.path.realpath(FileHandler.join(os.getcwd(), file_path))

        return os.path.realpath(file_path)

    def __call__(self, parser_in, namespace, values, option_string=None):
        """
        Inherited __call__ method from argparse.Action.

        Args:
            parser_in (ArgumentParser): Passed-in argument parser.
            namespace (Namespace): Namespace object to hold arguments.
            values (object): Argument values with type depending on argument definition.
            option_string (str): Optional string for specific argument name. Default: None.
        """
        summary_dir = self.check_path(values)

        setattr(namespace, self.dest, summary_dir)

class OutputDirAction(argparse.Action):
    """File directory action class definition."""

    def __call__(self, parser_in, namespace, values, option_string=None):
        """
        Inherited __call__ method from argparse.Action.

        Args:
            parser_in (ArgumentParser): Passed-in argument parser.
            namespace (Namespace): Namespace object to hold arguments.
            values (object): Argument values with type depending on argument definition.
            option_string (str): Optional string for specific argument name. Default: None.
        """
        output = DirAction.check_path(values)

        setattr(namespace, self.dest, output)


class Command(BaseCommand):
    """Start mindinsight service."""

    name = 'parse_summary'
    description = 'Parse summary file'

    def add_arguments(self, parser):
        """
        Add arguments to parser.

        Args:
            parser (ArgumentParser): Specify parser to which arguments are added.
        """

        parser.add_argument(
            '--summary-dir',
            type=str,
            action=DirAction,
            default=os.path.realpath(os.getcwd()),
            help="""
                    Optional, specify path for summary file directory. 
                    Default directory is the current working directory.
                """)

        parser.add_argument(
            '--output',
            type=str,
            action=OutputDirAction,
            default=os.path.realpath(os.getcwd()),
            help="""
                    Optional, specify path for converted file directory. Default output 
                    directory is `output` folder in the current working directory.
                """)

    def run(self, args):
        """
        Execute for parse_summary command.

        Args:
            args (Namespace): Parsed arguments to hold customized parameters.
        """
        try:
            date_time = datetime.datetime.now().strftime('output_%Y%m%d_%H%M%S_%f')
            output_path = os.path.join(args.output, date_time)

            summary_dir = args.summary_dir
            if not self._check_dirpath(summary_dir):
                return

            summary_parser = _SummaryParser(summary_dir)
            summary_files = summary_parser.filter_files(os.listdir(summary_dir))

            if not summary_files:
                parse_summary_logger.error('Path %s has no summary file.', summary_dir)
                return

            summary_files = summary_parser.sort_files(summary_files)
            filename = summary_files[-1]

            summary_file = FileHandler.join(summary_dir, filename)

            if not (self._check_filepath(summary_file) and self._check_create_filepath(output_path)
                    and self._check_create_filepath(FileHandler.join(output_path, 'image'))):
                return

            eventparser = EventParser(summary_file, output_path)
            eventparser.parse()

        except Exception as ex:
            parse_summary_logger.error("Parse summary file failed, detail: %r.", str(ex))
            raise UnknownError(str(ex))

    @staticmethod
    def _check_filepath(filepath):
        """
        Check file path existence, accessible and available

        Args:
            filepath (str): File path.
        """
        if not os.path.isfile(filepath):
            parse_summary_logger.error('Summary file %s is not a valid file.', filepath)
            return False
        if not os.access(filepath, os.R_OK):
            parse_summary_logger.error('Path %s is not accessible, please check the file-authority.', filepath)
            return False
        return True

    @staticmethod
    def _check_dirpath(filepath):
        """
        Check file path existence, accessible and available

        Args:
            filepath (str): File path.
        """
        if os.path.exists(filepath):
            if not os.path.isdir(filepath):
                parse_summary_logger.error('Summary directory %s is not a valid directory.', filepath)
                return False
            if not os.access(filepath, os.R_OK | os.X_OK):
                parse_summary_logger.error('Path %s is not accessible, please check the file-authority.', filepath)
                return False
            return True
        parse_summary_logger.error('Summary directory %s not exists.', filepath)
        return False

    @staticmethod
    def _check_create_filepath(filepath):
        """
        Check file path existence, accessible and available, if not exist create the file

        Args:
            filepath (str): File path.
        """
        permissions = os.R_OK | os.W_OK | os.X_OK
        os.umask(permissions << 3 | permissions)
        if os.path.exists(filepath):
            parse_summary_logger.error('Path %s has already existed, please choose a new output path.', filepath)
            return False
        mode = permissions << 6
        os.makedirs(filepath, mode=mode)
        return True



# Copyright 2020 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""This module is aimed to deal with watchpoint commands."""
from queue import Queue

from mindinsight.debugger.common.exceptions.exceptions import DebuggerParamValueError, \
    DebuggerCreateWatchPointError, DebuggerUpdateWatchPointError, \
    DebuggerDeleteWatchPointError, DebuggerConditionUnavailableError
from mindinsight.debugger.common.log import LOGGER as log
from mindinsight.debugger.common.utils import ServerStatus, \
    Streams, is_cst_type, is_scope_type
from mindinsight.debugger.conditionmgr.condition import ConditionIdEnum, ConditionContext
from mindinsight.debugger.conditionmgr.recommender import get_basic_node_info
from mindinsight.debugger.stream_handler.watchpoint_handler import validate_watch_condition


class WatchpointOperator:
    """Watchpoint Operator."""

    def __init__(self, cache_store, condition_mgr):
        self._watchpoint_stream = cache_store.get_stream_handler(Streams.WATCHPOINT)
        self._multi_card_graph_stream = cache_store.get_stream_handler(Streams.GRAPH)
        self._metadata_stream = cache_store.get_stream_handler(Streams.METADATA)
        self._device_stream = cache_store.get_stream_handler(Streams.DEVICE)
        self._condition_mgr = condition_mgr

    def create_watchpoint(self, params):
        """
        Create watchpoint.

        Args:
            - watch_condition (dict): The watch condition. The format is like:

              .. code-block::

                  {
                      "id": "tensor_too_large",
                      "params": [{
                          "name": "abs_mean_gt",
                          "value": 1.1
                      }]
                  }

                    - id (str): Id of condition.
                    - params (list[dict]): The list of param for this condition.
                - watch_nodes (list[str]): The list of node names.
                - watch_point_id (int): The id of watchpoint.
                - search_pattern (dict): The search pattern.
                - graph_name (str): The relative graph_name of the watched node.

        Returns:
            dict, the id of new watchpoint and metadata info.
        """
        watch_condition = params.get('watch_condition')
        log.info("Received create watchpoint request. WatchCondition: %s", watch_condition)
        metadata_stream = self._metadata_stream
        if metadata_stream.state != ServerStatus.WAITING.value:
            log.error("Failed to create watchpoint as the MindSpore is not in waiting state.")
            raise DebuggerCreateWatchPointError(
                "Failed to create watchpoint as the MindSpore is not in waiting state.")
        self._validate_watch_condition(watch_condition)

        validate_watch_condition(self._condition_mgr, watch_condition)
        condition_id = watch_condition.get('id')
        condition = self._condition_mgr.get_condition(condition_id)
        condition_context = ConditionContext(metadata_stream.backend, metadata_stream.step)
        if not condition.is_available(condition_context):
            log.error("Failed to create watchpoint as the condition is not available.")
            raise DebuggerConditionUnavailableError(
                "Failed to create watchpoint as the condition is not available.")

        watch_nodes = get_basic_node_info(condition.supported_target_type.value, self._multi_card_graph_stream)
        watchpoint_stream = self._watchpoint_stream
        watch_point_id = watchpoint_stream.create_watchpoint(condition_mgr=self._condition_mgr,
                                                             watch_condition=watch_condition,
                                                             watch_nodes=watch_nodes,
                                                             watch_point_id=params.get('watch_point_id'),
                                                             device_amount=self._device_stream.device_amount)
        log.info("Create watchpoint %d", watch_point_id)

        metadata_stream.enable_recheck = watchpoint_stream.is_recheckable()
        res = metadata_stream.get(['state', 'enable_recheck'])
        res['id'] = watch_point_id
        return res

    def _validate_watch_condition(self, watch_condition):
        """Validate watch condition."""
        metadata_stream = self._metadata_stream
        if metadata_stream.backend == 'GPU' and watch_condition.get('id') == ConditionIdEnum.OPERATOR_OVERFLOW.value:
            log.error("GPU doesn't support overflow watch condition.")
            raise DebuggerParamValueError("GPU doesn't support overflow watch condition.")

    def update_watchpoint(self, params):
        """
        Update watchpoint.

        Args:
            params (dict): Params for update watchpoint.

                - watch_point_id (int): The id of watchpoint.
                - watch_nodes (list[str]): The list of node names.
                - mode (int): The update operator on nodes. 0 for remove nodes from watch nodes.
                  1 for add nodes to watch nodes.
                - search_pattern (dict): The search pattern.
                - graph_name (str): The relative graph_name of the watched node.
                - rank_id (int): The rank id.

        Returns:
            dict, the metadata info.
        """
        metadata_stream = self._metadata_stream
        if metadata_stream.state != ServerStatus.WAITING.value:
            log.error("Failed to update watchpoint as the MindSpore is not in waiting state.")
            raise DebuggerUpdateWatchPointError(
                "Failed to update watchpoint as the MindSpore is not in waiting state."
            )
        # validate parameter
        watchpoint_stream = self._watchpoint_stream
        watch_point_id = params.get('watch_point_id')
        watch_nodes = params.get('watch_nodes')
        if not watch_nodes or not watch_point_id:
            log.error("Invalid parameter for update watchpoint.")
            raise DebuggerParamValueError("Invalid parameter for update watchpoint.")
        watchpoint_stream.validate_watchpoint_id(watch_point_id)
        # get node basic info for watch nodes
        watch_nodes = self._get_watch_node_with_basic_info(
            node_names=params.get('watch_nodes'),
            search_pattern=params.get('search_pattern'),
            graph_name=params.get('graph_name'),
            rank_id=params.get('rank_id', 0))
        watchpoint_stream.update_watchpoint(watch_point_id, watch_nodes, params.get('mode'), params.get('rank_id', 0))
        metadata_stream.enable_recheck = watchpoint_stream.is_recheckable()
        log.info("Update watchpoint with id: %d", watch_point_id)
        return metadata_stream.get(['state', 'enable_recheck'])

    def _get_watch_node_with_basic_info(self, node_names, search_pattern=None, graph_name=None, rank_id=0):
        """
        Get watch node with basic info.

        Args:
            node_names (list[str]): A list of node names.
            search_pattern (dict): Get watch node with search pattern. Default: None
            graph_name (str): The relative graph_name of the watched node. Default: None.
            rank_id (int): The rank id.

        Returns:
            list[NodeBasicInfo], a list of node basic infos.
        """
        if not node_names:
            return []
        graph_name = self._multi_card_graph_stream.get_graph_handler_by_rank_id(rank_id).validate_graph_name(graph_name)
        if search_pattern is not None:
            watch_nodes = self._get_watch_nodes_by_search(node_names, search_pattern, graph_name, rank_id)
        else:
            watch_nodes = self._get_node_basic_infos(node_names, graph_name=graph_name, rank_id=rank_id)
        return watch_nodes

    def _get_watch_nodes_by_search(self, node_names, search_pattern, graph_name, rank_id):
        """
        Get watched leaf nodes by search name.

        Args:
            node_names (list[str]): A list of node names.
            search_pattern (dict): Get watch node with search pattern.

                - name (str): The name pattern.
                - node_category (str): The node_category.
            graph_name (str): The relative graph_name of the watched node.

        Returns:
            list[NodeBasicInfo], a list of node basic infos.
        """
        search_pattern['graph_name'] = graph_name
        search_nodes = self._multi_card_graph_stream.get_graph_handler_by_rank_id(rank_id).search_nodes(search_pattern)
        watch_node_names = set()
        for name in node_names:
            names = self._get_watch_names_by_search(search_nodes, name)
            watch_node_names.update(names)
        watch_node_info = self._get_node_basic_infos(watch_node_names, graph_name=graph_name)
        log.debug("Update nodes: %s", watch_node_info)

        return watch_node_info

    def _get_watch_names_by_search(self, search_nodes, target_node_name):
        """
        Get watch names according to search results.

        Args:
            search_nodes (dict): Search result.
                The format is like {'nodes': [<Search Node>]}. The <Search Node> format is like
                {'name': <UI node name>, 'type': <node type>, 'nodes': [<Search Node>]}
            target_node_name (str): Node name for UI.

        Returns:
            set[str], collection of names.
        """
        names = set()
        tmp_queue = Queue()
        tmp_queue.put(search_nodes)
        while not tmp_queue.empty():
            cur_node = tmp_queue.get()
            for node in cur_node.get('nodes'):
                node_name = node.get('name')
                if not target_node_name.startswith(node_name) or is_cst_type(node.get('type')):
                    continue
                if target_node_name == node_name:
                    self._add_leaf_node_collection(node, names)
                    return names
                tmp_queue.put(node)
        # the target node name is not in search nodes.
        log.debug("node %s is not in search nodes.")
        names.add(target_node_name)
        return names

    def _add_leaf_node_collection(self, node, names):
        """
        Get leaf node collection in search node object.

        Args:
            node (dict): Search Node object.
            names (set): Set of node names.
        """
        # if the node is leaf node
        if not node.get('nodes'):
            names.add(node.get('name'))
            return
        # traverse the search node object
        for sub_node in node.get('nodes'):
            self._add_leaf_node_collection(sub_node, names)

    def delete_watchpoint(self, watch_point_id=None):
        """
        Delete watchpoint.

        Args:
            watch_point_id (Union[None, int]): The id of watchpoint.
                If None, delete all watchpoints. Default: None.

        Returns:
            dict, the metadata info.
        """
        metadata_stream = self._metadata_stream
        if metadata_stream.state != ServerStatus.WAITING.value:
            log.error("Failed to delete watchpoint as the MindSpore is not in waiting state.")
            raise DebuggerDeleteWatchPointError(
                "Failed to delete watchpoint as the MindSpore is not in waiting state."
            )
        watchpoint_stream = self._watchpoint_stream
        watchpoint_stream.delete_watchpoint(watch_point_id)
        metadata_stream.enable_recheck = watchpoint_stream.is_recheckable()
        log.info("Delete watchpoint with id: %s", watch_point_id)
        return metadata_stream.get(['state', 'enable_recheck'])

    def _get_node_basic_infos(self, node_names, graph_name=None, rank_id=0):
        """
        Get watch node info according to node names.

        Args:
            node_names (Union[set[str], list[str]]): A collection of node names.
            graph_name (str): The relative graph_name of the watched node. Default: None.

        Returns:
            list[NodeBasicInfo], a list of basic watch nodes info.
        """
        if not node_names:
            return []
        graph_stream = self._multi_card_graph_stream.get_graph_handler_by_rank_id(rank_id)
        node_infos = []
        for node_name in node_names:
            node_info = graph_stream.get_node_basic_info(node_name, graph_name)
            if not is_cst_type(node_info.type):
                node_infos.append(node_info)

        return node_infos

    @staticmethod
    def add_set_cmd_detail(set_cmd, watchpoint_stream, multi_card_graph_stream):
        """Add watch node and watch condition into set_cmd."""
        watchpoint = watchpoint_stream.get_watchpoint_by_id(set_cmd.id)
        watch_nodes_for_devices = watchpoint.get_watch_nodes()
        leaf_watch_nodes_for_devices = {}
        for rank_id, watch_nodes in watch_nodes_for_devices.items():
            graph_stream = multi_card_graph_stream.get_graph_handler_by_rank_id(rank_id)
            leaf_watch_nodes = WatchpointOperator.expand_to_leaf_nodes(graph_stream, watch_nodes)
            leaf_watch_nodes_for_devices[rank_id] = leaf_watch_nodes
        watchpoint.add_set_cmd_detail(set_cmd, leaf_watch_nodes_for_devices)

    @staticmethod
    def expand_to_leaf_nodes(graph_stream, watch_nodes):
        """
        Get all leaf node basic info according to watch nodes.

        Args:
            graph_stream (GraphHandler): Graph handler.
            watch_nodes (list[NodeBasicInfo]): The list of watch node basic infos.

        Returns:
            list[NodeBasicInfo], expanded leaf basic node infos.
        """
        leaf_watch_nodes = []
        for node in watch_nodes:
            if is_scope_type(node.type):
                pure_node_name = ''
                if len(node.name.split('/')) > 1:
                    graph_name, pure_node_name = node.name.split('/', 1)
                else:
                    graph_name = node.name
                search_node_infos = graph_stream.get_node_basic_info_by_scope(pure_node_name, graph_name=graph_name)
                search_node_infos = [node_info for node_info in search_node_infos if not is_cst_type(node_info.type)]
                leaf_watch_nodes.extend(search_node_infos)
            else:
                leaf_watch_nodes.append(node)
        return leaf_watch_nodes



# Copyright 2020-2021 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Profiler error code and messages."""
from enum import unique, Enum

from mindinsight.utils.constant import ProfilerMgrErrors


_GENERAL_MASK = 0b00001 << 7
_PARSER_MASK = 0b00010 << 7
_ANALYSER_MASK = 0b00011 << 7


@unique
class ProfilerErrors(ProfilerMgrErrors):
    """Profiler error codes."""
    # general error code
    PARAM_VALUE_ERROR = 0 | _GENERAL_MASK
    PATH_ERROR = 1 | _GENERAL_MASK
    PARAM_TYPE_ERROR = 2 | _GENERAL_MASK
    DIR_NOT_FOUND_ERROR = 3 | _GENERAL_MASK
    FILE_NOT_FOUND_ERROR = 4 | _GENERAL_MASK
    IO_ERROR = 5 | _GENERAL_MASK
    FIELD_MISSED_ERROR = 6 | _GENERAL_MASK

    # parser error code
    DEVICE_ID_MISMATCH_ERROR = 0 | _PARSER_MASK
    RAW_FILE_ERROR = 1 | _PARSER_MASK
    STEP_NUM_NOT_SUPPORTED_ERROR = 2 | _PARSER_MASK
    JOB_ID_MISMATCH_ERROR = 3 | _PARSER_MASK

    # analyser error code
    COLUMN_NOT_EXIST_ERROR = 0 | _ANALYSER_MASK
    ANALYSER_NOT_EXIST_ERROR = 1 | _ANALYSER_MASK
    DEVICE_ID_ERROR = 2 | _ANALYSER_MASK
    OP_TYPE_ERROR = 3 | _ANALYSER_MASK
    GROUP_CONDITION_ERROR = 4 | _ANALYSER_MASK
    SORT_CONDITION_ERROR = 5 | _ANALYSER_MASK
    FILTER_CONDITION_ERROR = 6 | _ANALYSER_MASK
    COLUMN_NOT_SUPPORT_SORT_ERROR = 7 | _ANALYSER_MASK
    PIPELINE_OP_NOT_EXIST_ERROR = 8 | _ANALYSER_MASK
    UNSUPPORTED_PARALLEL_TYPE_ERROR = 9 | _ANALYSER_MASK
    WRONG_PARALLEL_STRATEGY_DATA_ERROR = 10 | _ANALYSER_MASK
    NOT_FOUND_PARALLEL_STRATEGY_DATA_ERROR = 11 | _ANALYSER_MASK
    PARSE_PARALLEL_STRATEGY_DATA_ERROR = 12 | _ANALYSER_MASK
    WRONG_PARALLEL_STRATEGY_DATA_NUM_ERROR = 13 | _ANALYSER_MASK


@unique
class ProfilerErrorMsg(Enum):
    """Profiler error messages."""
    # general error msg
    PARAM_VALUE_ERROR = 'Param value error. {}'
    PATH_ERROR = 'Path error. {}'
    PARAM_TYPE_ERROR = 'Param type error. {}'
    DIR_NOT_FOUND_ERROR = 'The dir <{}> not found.'
    FILE_NOT_FOUND_ERROR = 'The file <{}> not found.'
    IO_ERROR = 'Read or write file fail.'

    # parser error msg
    DEVICE_ID_MISMATCH_ERROR = 'The device ID mismatch.'
    RAW_FILE_ERROR = 'Raw file error. {}'
    STEP_NUM_NOT_SUPPORTED_ERROR = 'The step num must be in {}'
    JOB_ID_MISMATCH_ERROR = 'The job id in the parameter is not the same as ' \
                            'in the training trace file. '

    # analyser error msg
    COLUMN_NOT_EXIST_ERROR = 'The column {} does not exist.'
    ANALYSER_NOT_EXIST_ERROR = 'The analyser {} does not exist.'
    DEIVICE_ID_ERROR = 'The device_id in search_condition error, {}'
    FILTER_CONDITION_ERROR = 'The filter_condition in search_condition error, {}'
    OP_TYPE_ERROR = 'The op_type in search_condition error, {}'
    GROUP_CONDITION_ERROR = 'The group_condition in search_condition error, {}'
    SORT_CONDITION_ERROR = 'The sort_condition in search_condition error, {}'
    COLUMN_NOT_SUPPORT_SORT_ERROR = 'The column {} does not support to sort.'
    PIPELINE_OP_NOT_EXIST_ERROR = 'The minddata pipeline operator {} does not exist.'
    UNSUPPORTED_PARALLEL_TYPE_ERROR = 'This parallel type is not supported. Detail: {}'
    WRONG_PARALLEL_STRATEGY_DATA_ERROR = "The provided parallel strategy data is wrong. Detail: {}"
    NOT_FOUND_PARALLEL_STRATEGY_DATA_ERROR = "Can not find any parallel strategy data."
    PARSE_PARALLEL_STRATEGY_DATA_ERROR = "Parse parallel strategy data failed. Detail: {}"
    WRONG_PARALLEL_STRATEGY_DATA_NUM_ERROR = "The num of parallel data is not match the rank size"



# Copyright 2020-2021 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Implement the debugger data cache manager."""
import sys

from mindinsight.debugger.common.log import LOGGER as log
from mindinsight.debugger.common.utils import Streams
from mindinsight.debugger.stream_handler import EventHandler, MetadataHandler, MultiCardGraphHandler, \
    MultiCardTensorHandler, WatchpointHandler, MultiCardWatchpointHitHandler
from mindinsight.debugger.stream_handler.device_handler import DeviceHandler
from mindinsight.debugger.stream_handler.graph_history_handler import GraphHistoryHandler

STREAM_HANDLER_MAP = {
    Streams.COMMAND.value: EventHandler,
    Streams.DATA.value: EventHandler,
    Streams.METADATA.value: MetadataHandler,
    Streams.GRAPH.value: MultiCardGraphHandler,
    Streams.TENSOR.value: MultiCardTensorHandler,
    Streams.WATCHPOINT.value: WatchpointHandler,
    Streams.WATCHPOINT_HIT.value: MultiCardWatchpointHitHandler,
    Streams.DEVICE.value: DeviceHandler,
    Streams.GRAPH_HISTORY.value: GraphHistoryHandler
}


class DebuggerCache:
    """The debugger data cache manager."""

    def __init__(self):
        self._stream_handler = {}

    def initialize(self):
        """Initialize the stream handlers."""
        self._stream_handler = {}
        for mode, stream_class in STREAM_HANDLER_MAP.items():
            self._stream_handler[mode] = stream_class()

    def clean(self):
        """Clean cache for all stream."""
        for _, stream_handler in self._stream_handler.items():
            stream_handler.clean()

    def get_stream_handler(self, mode):
        """
        Get the stream handler object.

        Args:
            mode (Streams): The type of stream handler.

        Returns:
            StreamHandlerBase, the stream handler object.
        """
        return self._stream_handler.get(mode.value)

    def _get(self, mode, pos):
        """
        Get updated data or command from cache.

        Args:
            mode (Streams): The type of info. `Streams.DATA` or `Streams.COMMAND`.
            pos (int): The index of info.

        Returns:
            object, the pos-th message about `mode` type of info.
        """
        stream_handler = self.get_stream_handler(mode)

        return stream_handler.get(pos)

    def _put(self, mode, value):
        """
        Set updated data or command from cache.

        Args:
            mode (Streams): The type of info. `Streams.DATA` or `Streams.COMMAND`.
            value (object): The info to be record in cache.
        """
        stream_handler = self.get_stream_handler(mode)

        return stream_handler.put(value)

    def get_command(self, pos):
        """
        Get the pos-th command in command stream.

        Args:
            pos (int): The index of command.

        Returns:
            int, the position of next message.
            EventReply, the command object.
        """
        content = self._get(Streams.COMMAND, pos)
        next_pos = content.get('metadata').get('pos')
        reply = content.get('cmd')
        return next_pos, reply

    def put_command(self, cmd):
        """
        Set command to command stream.

        Args:
            cmd (EventReply): The command EventReply.
        """
        log.debug("Set command %s", cmd)
        return self._put(Streams.COMMAND, {'cmd': cmd})

    def has_command(self, pos):
        """Judge if the number of command is no less than `pos`."""
        event = self.get_stream_handler(Streams.COMMAND).has_pos(pos)

        return event

    def clean_command(self):
        """Clean command queue."""
        self.get_stream_handler(Streams.COMMAND).clean()
        log.debug("Clean command.")

    def clean_data(self):
        """Clean command queue."""
        self.get_stream_handler(Streams.DATA).clean()
        log.debug("Clean data queue.")

    def get_data(self, pos):
        """
        Get updated data from data stream.

        Args:
            pos (int): The index of data.

        Returns:
            object, updated data_value.
        """
        return self._get(Streams.DATA, pos)

    def put_data(self, value):
        """
        Set updated data to data stream.

        Args:
            value (dict): The updated data.
        """
        log.debug("Set <%d> bytes data", sys.getsizeof(value))
        return self._put(Streams.DATA, value)



# Copyright 2020 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""
Data Loader Generator.

This module generate loaders from summary logs.
"""
import os
from mindinsight.datavisual.common.log import logger
from mindinsight.datavisual.common.exceptions import TrainJobNotExistError
from mindinsight.datavisual.data_access.file_handler import FileHandler
from mindinsight.datavisual.data_transform.data_loader import DataLoader
from mindinsight.datavisual.data_transform.loader_generators.loader_generator import MAX_DATA_LOADER_SIZE
from mindinsight.datavisual.data_transform.loader_generators.loader_struct import LoaderStruct
from mindinsight.datavisual.data_transform.loader_generators.loader_generator import LoaderGenerator
from mindinsight.datavisual.data_transform.summary_watcher import SummaryWatcher
from mindinsight.utils.exceptions import ParamValueError
from mindinsight.utils.exceptions import PathNotExistError


class DataLoaderGenerator(LoaderGenerator):
    """
    DataLoaderGenerator generate a loader_dict of loader from summary logs.

    Each loader helps deal the data of the events.
    It helps DataManager to generate loaders.
    """
    def __init__(self, summary_path):
        """
        Init DataLoaderGenerator.

        Args:
            summary_path (str): A directory path, e.g. '/data/ImageNet/'.
        """
        self._summary_path = self._check_and_normalize_summary_path(summary_path)
        self._summary_watcher = SummaryWatcher()

    def register_folder_analyzer(self, analyzer):
        """Register folder analyzer."""
        self._summary_watcher.register_folder_analyzer(analyzer)

    def _check_and_normalize_summary_path(self, summary_path):
        """
        Check and normalize summary path.

        Args:
            summary_path (str): A directory path, e.g. '/data/ImageNet/'.

        Returns:
            str, normalized summary path.

        """
        if summary_path is None:
            logger.warning("Summary path is None. It will not init data loader generator.")
            raise ParamValueError("Summary path is None.")

        summary_path = os.path.realpath(summary_path)

        return summary_path

    def generate_loaders(self, loader_pool, summaries_info):
        """
        Generate loader from summary path, if summary path is empty, will return empty list.

        Args:
            loader_pool (dict[str, LoaderStruct]): Current loader pool in data_manager.
            summaries_info (list): Summaries info list.

        Returns:
            dict[str, LoaderStruct], a dict of `Loader`.
        """
        loader_dict = {}

        if not FileHandler.exists(self._summary_path):
            logger.warning("Summary path does not exist. It will not start loading events data. "
                           "Current path is %r.", self._summary_path)
            return loader_dict

        dir_map_mtime_dict = {}
        min_modify_time = None

        for item in summaries_info:
            relative_path = item.get("relative_path")
            current_dir = FileHandler.join(self._summary_path, relative_path)
            dataloader = DataLoader(current_dir)

            if not dataloader.has_valid_files():
                logger.debug("Can not find valid train log file in folder %s , "
                             "will ignore.", relative_path)
                continue

            modify_time = item.get("update_time").timestamp()

            # if loader exists in loader pool and newer time, update its time
            loader_id = self._generate_loader_id(relative_path)
            loader = loader_pool.get(loader_id)
            if loader is not None and loader.latest_update_time > modify_time:
                modify_time = loader.latest_update_time

            if not min_modify_time:
                # The first load, init min modify time
                min_modify_time = modify_time

            # We need to find `MAX_DATA_LOADER_SIZE` newly modified folders.
            if len(dir_map_mtime_dict) < MAX_DATA_LOADER_SIZE:
                if modify_time < min_modify_time:
                    min_modify_time = modify_time
                dir_map_mtime_dict.update({relative_path: modify_time})

            else:
                if modify_time >= min_modify_time:
                    dir_map_mtime_dict.update({relative_path: modify_time})

        sorted_dir_tuple = sorted(dir_map_mtime_dict.items(),
                                  key=lambda d: d[1])[-MAX_DATA_LOADER_SIZE:]

        for relative_path, modify_time in sorted_dir_tuple:
            loader_id = self._generate_loader_id(relative_path)
            loader = self._generate_loader_by_relative_path(relative_path)
            loader_dict.update({loader_id: loader})

        return loader_dict

    def _generate_loader_by_relative_path(self, relative_path):
        """
        Generate loader by relative path.

        Args:
            relative_path (str): Relative path of a summary directory, e.g. './log1'.

        Returns:
            dict[str, LoaderStruct], a dict of `Loader`.
        """
        current_dir = os.path.realpath(FileHandler.join(self._summary_path, relative_path))
        data_loader = DataLoader(current_dir)
        loader_id = self._generate_loader_id(relative_path)
        loader = LoaderStruct(loader_id=loader_id,
                              name=self._generate_loader_name(relative_path),
                              path=current_dir,
                              latest_update_time=FileHandler.file_stat(current_dir).mtime,
                              data_loader=data_loader)
        return loader

    def _generate_loader_id(self, relative_path):
        """
        Generate loader id from relative path.

        Args:
            relative_path (str): Relative path of a summary directory, e.g. './log1'.

        Returns:
            str, loader_id for `Loader`.

        """
        loader_id = relative_path
        return loader_id

    def _generate_loader_name(self, relative_path):
        """
        Generate loader name from relative path.

        Args:
            relative_path (str): Relative path of a summary directory, e.g. './log1'.

        Returns:
            str, loader_name for `Loader`.

        """
        loader_name = relative_path
        return loader_name

    def _get_relative_path_from_train_id(self, train_id):
        """
        Get relative from train_id.

        Args:
            train_id (str): Train ID of a summary directory, e.g. './log1'.

        Returns:
            str, relative path of `Loader`.

        """
        relative_path = train_id

        return relative_path

    def check_train_job_exist(self, train_id):
        """
        Check if train job exists.

        Args:
            train_id (str): Train ID of a summary directory, e.g. './log1'.

        Returns:
            bool, if train job exists, return True.

        """
        if not self._is_train_id_valid(train_id):
            return False

        relative_path = self._get_relative_path_from_train_id(train_id)
        if self._summary_watcher.is_summary_directory(self._summary_path, relative_path):
            return True

        return False

    def _is_train_id_valid(self, train_id):
        """
        Check if train_id is valid.

        Args:
            train_id (str): Train ID of a summary directory, e.g. './log1'.

        Returns:
            bool, if train id is valid, return True.

        """
        if not train_id.startswith('./'):
            logger.warning("The train_id does not start with './'.")
            return False
        if len(train_id.split("/")) > 2:
            logger.warning("The train_id contains multiple '/'.")
            return False
        return True

    def generate_loader_by_train_id(self, train_id):
        """
        Generate loader by train_id.

        Args:
            train_id (str): Train ID of a summary directory, e.g. './log1'.

        Returns:
            dict[str, LoaderStruct], a dict of `Loader`.

        """
        relative_path = self._get_relative_path_from_train_id(train_id)
        try:
            loader = self._generate_loader_by_relative_path(relative_path)
        except PathNotExistError as ex:
            raise TrainJobNotExistError(str(ex))

        return loader



# Copyright 2019 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Lineage restful api."""
import json

from flask import Blueprint, jsonify, request

from mindinsight.conf import settings
from mindinsight.datavisual.data_transform.data_manager import DATA_MANAGER
from mindinsight.lineagemgr.cache_item_updater import update_lineage_object
from mindinsight.lineagemgr.common.validator.validate import validate_train_id
from mindinsight.lineagemgr.model import filter_summary_lineage
from mindinsight.utils.exceptions import MindInsightException, ParamValueError

BLUEPRINT = Blueprint("lineage", __name__, url_prefix=settings.URL_PATH_PREFIX+settings.API_PREFIX)


@BLUEPRINT.route("/lineagemgr/lineages", methods=["POST"])
def get_lineage():
    """
    Get lineage.

    Returns:
        str, the lineage information.

    Raises:
        MindInsightException: If method fails to be called.
        ParamValueError: If parsing json data search_condition fails.

    Examples:
        >>> POST http://xxxx/v1/mindinsight/lineagemgr/lineages
    """
    search_condition = request.stream.read()
    try:
        search_condition = json.loads(search_condition if search_condition else "{}")
    except Exception:
        raise ParamValueError("Json data parse failed.")

    lineage_info = _get_lineage_info(search_condition=search_condition)

    return jsonify(lineage_info)


def _get_lineage_info(search_condition):
    """
    Get lineage info for dataset or model.

    Args:
        search_condition (dict): Search condition.

    Returns:
        dict, lineage info.

    Raises:
        MindInsightException: If method fails to be called.
    """
    try:
        lineage_info = filter_summary_lineage(data_manager=DATA_MANAGER, search_condition=search_condition)

    except MindInsightException as exception:
        raise MindInsightException(exception.error, exception.message, http_code=400)

    return lineage_info


@BLUEPRINT.route("/lineagemgr/lineages", methods=["PUT"])
def update_lineage():
    """
    Get lineage.

    Returns:
        str, update the lineage information about cache and tag.

    Raises:
        MindInsightException: If method fails to be called.

    Examples:
        >>> PUT http://xxxx/v1/mindinsight/lineagemgr/lineages?train_id=./run1
    """
    train_id = request.args.get('train_id')
    added_info = request.json
    if not isinstance(added_info, dict):
        raise ParamValueError("The request body should be a dict.")

    update_lineage_object(DATA_MANAGER, train_id, added_info)

    return jsonify({"status": "success"})


@BLUEPRINT.route("/datasets/dataset_graph", methods=["GET"])
def get_dataset_graph():
    """
    Get dataset graph.

    Returns:
        str, the dataset graph information.

    Raises:
        MindInsightException: If method fails to be called.
        ParamValueError: If summary_dir is invalid.

    Examples:
        >>> GET http://xxxx/v1/mindinsight/datasets/dataset_graph?train_id=xxx
    """

    train_id = request.args.get('train_id')
    validate_train_id(train_id)
    search_condition = {
        'summary_dir': {
            'in': [train_id]
        }
    }
    result = {}
    try:
        objects = filter_summary_lineage(data_manager=DATA_MANAGER, search_condition=search_condition).get('object')
    except MindInsightException as exception:
        raise MindInsightException(exception.error, exception.message, http_code=400)

    if objects:
        lineage_obj = objects[0]
        dataset_graph = lineage_obj.get('dataset_graph')

        if dataset_graph:
            result.update({'dataset_graph': dataset_graph})
            result.update({'summary_dir': lineage_obj.get('summary_dir')})

    return jsonify(result)


def init_module(app):
    """
    Init module entry.

    Args:
        app (Flask): The application obj.
    """
    app.register_blueprint(BLUEPRINT)



# Copyright 2019 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""base_notebook"""

class BaseNotebook:
    """
    Base notebook.
    """
    def __init__(self, **kwargs):
        pass

    def start(self, args_string, **kwargs):
        kwargs = kwargs
        return args_string

    def stop(self, port, **kwargs):
        pass

    def handle(self, **kwargs):
        pass



# Copyright 2021 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Query module."""

import re


def construct_filter_func(query_str, case_sensitive=False, use_regex=False):
    """Get filter func."""
    if use_regex:
        if case_sensitive:
            func = lambda x: bool(re.search(query_str, str(x)))
        else:
            func = lambda x: bool(re.search(query_str, str(x), flags=re.I))
    else:
        if case_sensitive:
            func = lambda x: str(x).find(query_str) > -1
        else:
            func = lambda x: str(x).lower().find(query_str.lower()) > -1
    return func


class StackQuery:
    """Stack query."""

    def __init__(self, operators):
        self.operators = operators

    def all(self):
        """
        Retrieve all operators.

        Returns:
            list, all operators.
        """
        return self.operators

    def get(self, index=0):
        """
        Retrieve one operator.

        Args:
            index (int): Operator index, default is 0.

        Returns:
            Operator, single operator.
        """
        if 0 <= index < len(self.operators):
            return self.operators[index]
        return None

    def filter(self, qs, case_sensitive=False, use_regex=False):
        """
        Filter operators with query.

        Args:
            qs (str): Query string.
            case_sensitive (bool): Indicates if case is sensitive. Default is False.
            use_regex (bool): Indicates if qs is regex. Default is False.

        Returns:
            StackQuery, cloned object.
        """
        func = construct_filter_func(qs, case_sensitive, use_regex)
        operators = []
        for operator in self.operators:
            for res in map(func, operator.stack):
                if res:
                    operators.append(operator)
                    break

        return self.clone(operators)

    def clone(self, operators):
        """
        Clone query object.

        Returns:
            StackQuery, cloned object.
        """
        return StackQuery(operators)
